@@ -1,5 +1,5 @@
-import os
 import logging
+import os
 from argparse import ArgumentParser
 
 
@@ -24,7 +24,5 @@ else:
 
 
 # Make sure directories exist
-if not os.path.isdir(args.data_dir):
-    os.makedirs(args.data_dir, exist_ok=True)
-if not os.path.isdir(args.contests_dir):
-    os.makedirs(args.contests_dir, exist_ok=True)
+os.makedirs(args.data_dir, exist_ok=True)
+os.makedirs(args.contests_dir, exist_ok=True)
