@@ -1,8 +1,6 @@
-import os
-
 from ocs.data import about_data, contest_data, problem_data
 from ocs.db import con, cur
-from ocs.user import hash, make_token, save_token
+from ocs.user import check_password, create_user, modify_user, delete_user, make_token, save_token, check_token
 from ocs.problem import process
 
 
@@ -25,7 +23,7 @@ def solves(contest, problem=None):
 
     if problem is None:
         solves = {}
-        for problem in contest_data[contest]:
+        for problem in contest_data[contest][problem]:
             solves[problem] = cur.execute(
                 'SELECT COUNT(*) FROM "' + contest + '_status" WHERE "' + problem + '" = 202').fetchone()[0]
         return (200, solves)
@@ -43,31 +41,39 @@ def history(contest, problem=None):
             + contest + '_submissions" WHERE problem = ?', (problem,)).fetchall())
 
 
-def register(name, email, username, password):
-    """Register a new user"""
-
-    if cur.execute('SELECT Count(*) FROM users WHERE username = ?', (username,)).fetchone()[0] != 0:
-        return 409
-    cur.execute('INSERT INTO users VALUES (?, ?, ?, ?)', (name, email, username, hash(password, os.urandom(32))))
-    con.commit()
-    return 201
+def user(name, email, username, password):
+    """Manage user accounts"""
+    
+    if cur.execute('SELECT Count(*) FROM users WHERE username = ?', (username,)).fetchone()[0] == 0:
+        create_user(name, email, username, password)
+        return 201
+    
+    ret = check_password(username, password)
+    if ret != 200:
+        return ret
+    if name == 'DELETE':
+        delete_user(name, email, username, password)
+    else:
+        modify_user(name, email, username, password)    
+    return 200
 
 
 def authenticate(username, password, server):
     """Verify username and password"""
     
-    users = cur.execute('SELECT * FROM users WHERE username = ?', (username,)).fetchall()
-    if len(users) == 0:
-        return 404  # Username not found
-    if users[0][3] == hash(password, users[0][3][:32]):
+    ret = check_password(username, password)
+    if ret == 200:
         return (200, make_token(username, server))  # Make token
-    return 403  # Incorrect password
+    return ret
 
 
 def authorize(username, token):
-    """Save authentication token"""
+    """Receive authentication token"""
 
-    save_token(username, token)
+    if len(token) == 32:
+        save_token(username, token)
+    else:
+        check_token(username, token)
     return 200
 
 
