@@ -1,19 +1,29 @@
 import logging
-from json import loads, dumps
+from datetime import datetime
 from http.server import BaseHTTPRequestHandler
 from inspect import signature
-from datetime import datetime
+from json import loads, dumps
+
+from requests import post
 
 import ocs.request
 from ocs.data import about_data, contest_data
-from ocs.user import check_token
+from ocs.user import combine_token
 
 
-class server(BaseHTTPRequestHandler):
+class Server(BaseHTTPRequestHandler):
     """Main HTTP server"""
 
+    def CORS(self):
+        """Handle CORS"""
+
+        self.send_header('Access-Control-Allow-Origin', '*')
+        self.send_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
+        self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
+
     def send(self, result):
         """Send HTTP response"""
+        
         if isinstance(result, int):
             code, body = result, None
         else:
@@ -22,9 +32,7 @@ class server(BaseHTTPRequestHandler):
         logging.debug(body)
 
         self.send_response(code)  # Send status code
-        self.send_header('Access-Control-Allow-Origin', '*')
-        self.send_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
-        self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
+        self.CORS()
         if body is None:
             self.end_headers()
         else:
@@ -40,10 +48,9 @@ class server(BaseHTTPRequestHandler):
         if 'type' not in body:
             return 400  # Bad request
         if body['type'] not in dir(ocs.request):
-            return 500  # Not implemented
+            return 501  # Not implemented
 
         # Check if all required parameters are in the request
-        # TODO: Make this more robust
         parameters = str(signature(eval('ocs.request.' + body['type'])))[1:-1]
         for parameter in parameters.split():
             if 'None' in parameter:  # Optional parameter
@@ -57,9 +64,13 @@ class server(BaseHTTPRequestHandler):
 
         # Check token
         if 'token' in body and not body['type'] == 'authorize':
-            authorization = check_token(body['username'], body['token'])
-            if not authorization:
-                return 404  # Token not found
+            token = combine_token(body['username'], body['token'])
+            if token == 0:
+                return 401  # Token not found
+            authorization = post(('https://' if body['homeserver'].find('://') == -1 else '') + body['homeserver'],
+                json={'type': 'authorize', 'username': body['username'], 'token': token}).status_code
+            if authorization != 200:
+                return authorization
 
         # Check if contest exists
         if 'contest' in body and body['contest'] not in about_data['contests']:
@@ -76,21 +87,31 @@ class server(BaseHTTPRequestHandler):
         else:
             return eval('ocs.request.' + body['type'] + '(body["' + parameters.replace(', ', '"], body["') + '"])')
 
-    def do_OPTIONS(self):
-        """Handle CORS"""
-        
-        self.send_response(200)
-        self.send_header('Access-Control-Allow-Origin', '*')
-        self.send_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
-        self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
-        self.end_headers()
-
     def do_POST(self):
         """Handle POST requests"""
 
         # Decode request body
         content_length = int(self.headers['Content-Length'])
-        body = loads(self.rfile.read(content_length).decode('utf-8'))
+        try:
+            body = loads(self.rfile.read(content_length).decode('utf-8'))
+        except JSONDecodeError:
+            self.send(400)  # Bad request
         logging.debug(body)
 
         self.send(self.process(body))  # Process request and send back results
+    
+    def do_GET(self):
+        """Handle GET requests"""
+        
+        self.send_response(200)
+        self.CORS()
+        self.end_headers()
+        self.wfile.write('<html><body>Your <a href="https://github.com/LadueCS/OpenContest">\
+                         OpenContest</a> server is working!</body></html>'.encode('utf-8'))
+    
+    def do_OPTIONS(self):
+        """Handle OPTIONS requests"""
+        
+        self.send_response(200)
+        self.CORS()
+        self.end_headers()
